 varying highp vec2 v_TexCoordinate;
 uniform sampler2D u_Texture;
 uniform sampler2D u_ToneCurve2Texture;//color [0..255][0]
 //uniform sampler2D toneCurve3Texture;//color2 [0..255][0]
 
 uniform lowp float u_Ratio;
 uniform highp float u_AspectRatio;
 uniform highp float u_Angle;//radians
 
 uniform lowp float u_Alpha1;
 uniform lowp float u_Alpha2;
 
 uniform highp int u_GradType;
 uniform highp float u_Fade;
 
 uniform highp vec2 u_RightBottom; 
 
 const lowp float mc = 0.5;
 const mediump vec3 W = vec3(0.299, 0.587, 0.114);
 
 void main()
 {
     
     lowp vec4 base = texture2D(u_Texture, v_TexCoordinate);
     
     highp float lum = dot(base.rgb,W);
     
     lowp vec4 topColor = vec4(texture2D(u_ToneCurve2Texture, vec2(lum, 0.0)).rgb,u_Alpha1);
     
     lowp vec4 bottomColor = vec4(texture2D(u_ToneCurve2Texture, vec2(lum, 1.0/256.0)).rgb,u_Alpha2);

     //quotient
     lowp vec2 coords = v_TexCoordinate;
     
     coords.x = coords.x / u_RightBottom.x;
     coords.y = coords.y / u_RightBottom.y;
     
     highp float quotient = 1.0;
     
     //coords.x = coords.x*0.8+0.1;
     
     if(u_GradType==2){
         
         coords.x = 1.0-2.0*(abs((coords.x-mc)*cos(u_Angle)+(coords.y-mc)*sin(u_Angle)));//mirror
         
     } else if(u_GradType==1){
         
         lowp vec2 center = vec2(mc);
         
         coords.x = u_AspectRatio*coords.x;
         center.x = u_AspectRatio*center.x;
         
         highp float D = sqrt(1.0+u_AspectRatio*u_AspectRatio);
         
         coords.x = coords.x/D;
         coords.y = coords.y/D;
         center.x = center.x/D;
         center.y = center.y/D;
         
         coords.x = distance(center,coords)*2.0;//radial
         
     } else{
         
         coords.x = (coords.x-mc)*cos(u_Angle)+(coords.y-mc)*sin(u_Angle)+mc;//linear
         
     }
     
     if(u_Ratio > 0.0){
         coords.x = coords.x > 1.0-u_Ratio ? 1.0 : (coords.x)/(1.0-u_Ratio);
     } else if(u_Ratio < 0.0) coords.x = coords.x < -u_Ratio ? 0.0 : (coords.x+u_Ratio)/(1.0+u_Ratio);
     
     quotient = coords.x;
     
     lowp vec4 overlay = mix(topColor,bottomColor,quotient);
     
     overlay.rgb = overlay.rgb*overlay.a;//need for not black transparent colors, strange in simple texture transparent that is not need
     
     lowp vec4 blend;
     
     lowp float pa = base.a * (1.0 - overlay.a);//
     
     blend.r = overlay.r + base.r * pa;
     blend.g = overlay.g + base.g * pa;
     blend.b = overlay.b + base.b * pa;
     blend.a = overlay.a + pa;
     
     gl_FragColor = vec4(mix(base.rgb,blend.rgb,u_Fade), base.a);
     
     
 }